function Prelearning_ImagingCS
%% Prelearning_Imaing is for one photon prelearning imaging 
% 1:reward approaching; 2:reference cue; 0:active avoidance
% CSs and USs are in two blocks
% in CS block, three tones are randomly
% in US block, two USs are randomly
% SETUP
% > Connect the water valve in the box to Bpod Port#1.
% > Connect the air valve in the box to Bpod Port#2.
% > Lick: Bpod Port#3
% > Running: Wire 1 (input)
% > Imaging: Wire 1 (output)
% > BNC1: ArchT laser
% > BNC2: ChR2 laser

global BpodSystem
%% Define parameters
S = BpodSystem.ProtocolSettings; % Load settings chosen in launch manager into current workspace as a struct called S
if isempty(fieldnames(S))  % If settings file was an empty struct, populate struct with default settings
    S.RndFlag = 1; % Deploy the random trials or repeat a list of trials. 
    S.NonrandomMaxTrials = 20; %Total number of trials to run. This should not exceed the limit of the grapher (BpodSystem.PluginObjects.maxTrials)
    S.NonrandomSequence = [zeros(1,20),ones(1,20)]; % If nonrandom, sequence of the trials to run. 
   	S.BaselineTime = 10;
    S.arduinoPort = 'COM9'; % Port (USB) of the wheel run sensor Arduino
	S.wheelCirc = 46; % Circumference of the wheel (cm)
	S.wheelTicks = 1024; % Number of ticks recorded by the wheel per rotation.
	S.wheelMult = S.wheelCirc/S.wheelTicks; % Distance per tick (cm)
    %Attempt to connect to the Arduino run detector. 
	BpodSystem.PluginObjects.ardSerial = serial(S.arduinoPort);
    BPAS = BpodSystem.PluginObjects.ardSerial;
	BPAS.InputBufferSize = 16000; % read up to 2kb
	set(BPAS,'DataBits',8);
	set(BPAS,'StopBits',1);
	set(BPAS,'BaudRate',115200);
	set(BPAS,'Parity','none');
	fopen(BPAS);
	autoCleanup = onCleanup(@()fclose(BPAS));
    
    S.CueDelay = 5; % the time from cue to response
    S.ITI = 20;
    S.ITI_min =15; S.ITI_max = 25;
    S.SoundDuration = 5;
end

% Initialize parameter GUI plugin
% BpodParameterGUI('init', S);
% TotalRewardDisplay('init');

%% Define trials
if (~S.RndFlag && BpodSystem.PluginObjects.maxTrials < S.NonrandomMaxTrials)
    MaxTrials = BpodSystem.PluginObjects.maxTrials;
else
    MaxTrials = S.NonrandomMaxTrials;
end

if  S.RndFlag
    RandNumber = randperm(MaxTrials);
    TrialTypes =  mod(RandNumber,2);
else    
    disp('Using nonrandom trial list.')
    TrialTypes = [zeros(10,1);ones(10,1)];
end


BpodSystem.Data.TrialTypes = []; % The trial type of each trial completed will be added here.
BpodSystem.Data.ITI = [];
BpodSystem.Data.RunEvents = [];
BpodSystem.Data.RunSpeed = {};
BpodSystem.Data.CurrentTrialRunSpeed = zeros(10000,2);
BpodSystem.Data.CurrentTrialRunEvents = 0;
%% Initialize plots
BpodSystem.ProtocolFigures.SideOutcomePlotFig = figure('Position', [500 200 800 300],'name','Outcome plot','numbertitle','off', 'MenuBar', 'none', 'Resize', 'off');
BpodSystem.GUIHandles.SideOutcomePlot = axes('Position', [.075 .3 .89 .6]);
% BpodSystem.ProtocolFigures.LickPlotFig = figure('Position', [600 200 600 200],'name','Licking','numbertitle','off', 'MenuBar', 'none', 'Resize', 'off');
ThreeOutcomePlot_pre(BpodSystem.GUIHandles.SideOutcomePlot,'init',TrialTypes);

% Set soft code handler to trigger sounds
BpodSystem.SoftCodeHandlerFunction = 'SoftCodeHandler_PlaySoundX';

SF = 192000; % Sound card sampling rate
SinWaveFreq1 = 4000;
sounddata1 = GenerateSineWave(SF, SinWaveFreq1, S.SoundDuration); % Sampling freq (hz), Sine frequency (hz), duration (s)

SinWaveFreq2 = 12000;
sounddata2 = GenerateSineWave(SF, SinWaveFreq2, S.SoundDuration); % Sampling freq (hz), Sine frequency (hz), duration (s)
% sounddata3 = (rand(1,SF*S.SoundDuration+1)*2) - 1;
% WidthOfFrequencies=1.5; NumberOfFrequencies=7; MeanSoundFreq4 = 6000; SoundRamping=0.2;
% sounddata4 = SoundGenerator(SF, MeanSoundFreq4, WidthOfFrequencies, NumberOfFrequencies, S.SoundDuration, SoundRamping);

% Program sound server
PsychToolboxSoundServer('init')
PsychToolboxSoundServer('Load', 1, 0.2*sounddata1);
PsychToolboxSoundServer('Load', 2, 0.4*sounddata2);
% PsychToolboxSoundServer('Load', 3, 0.6*sounddata3);
% PsychToolboxSoundServer('Load', 3, sounddata3);
% PsychToolboxSoundServer('Load', 4, sounddata4);

%% Main trial loop
T = timer('period',1,'executionmode','fixedrate','taskstoexecute',9999999);
T.TimerFcn = @(~,~)scriptMonitor();
T.Period = 1;
T.ExecutionMode = 'fixedRate';
start(T);
autoTimerCleanup = onCleanup(@()delete(timerfind));
for currentTrial = 1:MaxTrials
    S.GUI.currentTrail = currentTrial;
%     S = BpodParameterGUI('sync', S); % Sync parameters with BpodParameterGUI plugin
      
    switch TrialTypes(currentTrial) % Determine trial-specific state matrix fields
        case 0 % sound 1
            soundID = 1;
        case 1 % sound 2
            soundID = 2;
		
    end
    
        sma = NewStateMatrix(); % Assemble state matrix
        
        %Global timer for synchronising events. Timer1: Sound, Timer2: Actions. 
        
        sma = SetGlobalTimer(sma,  'TimerID', 1, 'Duration',25, 'OnsetDelay', 0, 'channel','BNC2');
           
        sma = AddState(sma, 'Name', 'TrialStart', ...%1
            'Timer', S.BaselineTime,...
            'StateChangeConditions', {'Tup', 'StimulusDeliver'},...
            'OutputActions', {'GlobalTimerTrig', 1});
        
        sma = AddState(sma, 'Name', 'StimulusDeliver', ...%2
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'CueDelay'},...
            'OutputActions', {'SoftCode', soundID});
        sma = AddState(sma, 'Name', 'CueDelay', ...%3
            'Timer', S.CueDelay,...
            'StateChangeConditions', {'Tup', 'EndTime'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'EndTime', ...%15
            'Timer', S.BaselineTime,...
            'StateChangeConditions', {'Tup', 'ITI'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'ITI', ...%16
            'Timer', S.ITI,...
            'StateChangeConditions', {'Tup', 'exit'},...
            'OutputActions', {});
     
% 	BpodSystem.ProtocolFigures.CurrentTrialStartTime = datenum(datetime)*86400-BpodSystem.ProtocolFigures.StartTime;
    BpodSystem.Data.CurrentTrialRunSpeed = zeros(10000,2);
    BpodSystem.Data.CurrentTrialRunEvents = 0;
    SendStateMatrix(sma);
    RawEvents = RunStateMatrix;
	BpodSystem.Data.CurrentTrialLickTimes = [];
    if ~isempty(fieldnames(RawEvents)) % If trial data was returned
		%disp(RawEvents);
		RE = RawEvents.Events;
        %disp(RE);
		RES = RawEvents.States;
        %disp(RES);
		REST = RawEvents.StateTimestamps(2:end);
		%disp(REST);
		RET = RawEvents.EventTimestamps;
        %disp(RET);
		%Find if any timestamps contain a reward deliver (event 13) and
		%record that as the current trial reward delivery timestamp.
		RESR = (RES==13);
		%disp(RESR);
% 		BpodSystem.Data.CurrentTrialRewardTime = round(BpodSystem.ProtocolFigures.CurrentTrialStartTime+sum(RESR.*REST));
		%Filter the timestamps for licks (In3Up, 53/30 depending on computer) 
		%and turn them into a list.
		REE = (RE==32);
% 		BpodSystem.Data.CurrentTrialLickTimes = round(nonzeros(REE.*RET)+BpodSystem.ProtocolFigures.CurrentTrialStartTime);
        %disp(BpodSystem.Data.CurrentTrialLickTimes)
        BpodSystem.Data = AddTrialEvents(BpodSystem.Data,RawEvents); % Computes trial events from raw data
        BpodSystem.Data.TrialSettings(currentTrial) = S; % Adds the settings used for the current trial to the Data struct (to be saved after the trial ends)
        BpodSystem.Data.TrialTypes(currentTrial) = TrialTypes(currentTrial); % Adds the trial type of the current trial to data
        BpodSystem.Data.ITI(currentTrial) = S.ITI;
        %Outcome
                
        %Save the run timings and speeds
        UpdateThreeOutcomePlot_pre(TrialTypes, BpodSystem.Data);
        BpodSystem.Data.RunEvents(currentTrial) = BpodSystem.Data.CurrentTrialRunEvents;
        BpodSystem.Data.RunSpeed{currentTrial} = BpodSystem.Data.CurrentTrialRunSpeed;
        SaveBpodSessionData; % Saves the field BpodSystem.Data to the current data file
    end
    HandlePauseCondition; % Checks to see if the protocol is paused. If so, waits until user resumes.
    if BpodSystem.Status.BeingUsed == 0
        return
    end
end
    function UpdateThreeOutcomePlot_pre(~, Data)
	Outcomes = ones(1,Data.nTrials);
	ThreeOutcomePlot_pre(BpodSystem.GUIHandles.SideOutcomePlot,'update',Data.nTrials+1,TrialTypes,Outcomes);
end

end

function scriptMonitor()
	%Runs every second to monitor background events. 
	global BpodSystem
    S = BpodSystem.ProtocolSettings;
	%Reads the data from the Arduino serial port. 
    BPAS = BpodSystem.PluginObjects.ardSerial;
	dataSize = get(BPAS,'BytesAvailable');
	if (dataSize>0)
		inASCII = fread(BPAS,dataSize);
		inData = transpose(char(inASCII));
		inData = strsplit(inData);
		inData = inData(1:end-1); % Remove the last newline from the array
		for k=1:length(inData) % Convert the array to run speed / run distance and write it to the run speed array. 
			BpodSystem.Data.CurrentTrialRunSpeed(k+BpodSystem.Data.CurrentTrialRunEvents,1:2) = str2double(strsplit(inData{k},'A'));
        end
        BpodSystem.Data.CurrentTrialRunEvents = BpodSystem.Data.CurrentTrialRunEvents + length(inData);
		assignin('base', 'inString', BpodSystem.Data.CurrentTrialRunEvents);
	end
	TotalRewardDisplay('update');
end

