function PupilMeasurement
%% PupilMeasurement is for pupil/eye size measurement upon laser stimulation in box #1
% SETUP
% > Bpod BNC #1: Point Grey imaging
% > Bpod BNC #2: ChR2 stimulation
%  Xian Zhang email: xzhang@cshl.edu
global BpodSystem
%% Define parameters
S = BpodSystem.ProtocolSettings; % Load settings chosen in launch manager into current workspace as a struct called S
if isempty(fieldnames(S))  % If settings file was an empty struct, populate struct with default settings
    S.ImagingFlag = 1; % pointgrey imaging
    S.LaserStates = 1; % 0: No laser; 1: laser
    S.MaxTrials = 20; %Total number of trials to run. This should not exceed the limit of the grapher (BpodSystem.PluginObjects.maxTrials)
    S.GUI.BaselineTime = 10;
    S.GUI.LaserDuration = 5;
    S.GUI.ImagingDuration = 25;
    S.GUI.ITI = 10;
    
    S.arduinoPort = 'COM9'; % Port (USB) of the wheel run sensor Arduino
	S.wheelCirc = 46; % Circumference of the wheel (cm)
	S.wheelTicks = 1024; % Number of ticks recorded by the wheel per rotation.
	S.wheelMult = S.wheelCirc/S.wheelTicks; % Distance per tick (cm)
    %Attempt to connect to the Arduino run detector. 
	BpodSystem.PluginObjects.ardSerial = serial(S.arduinoPort);
    BPAS = BpodSystem.PluginObjects.ardSerial;
	BPAS.InputBufferSize = 16000; % read up to 2kb
	set(BPAS,'DataBits',8);
	set(BPAS,'StopBits',1);
	set(BPAS,'BaudRate',115200);
	set(BPAS,'Parity','none');
	fopen(BPAS);
	autoCleanup = onCleanup(@()fclose(BPAS));
 end

% Initialize parameter GUI plugin
BpodParameterGUI('init', S);
% TotalRewardDisplay('init');

%% Define trials
    MaxTrials = S.MaxTrials;
    TrialTypes = ones(MaxTrials,1);
 
BpodSystem.Data.TrialTypes = []; % The trial type of each trial completed will be added here.
BpodSystem.Data.ITI = [];
BpodSystem.Data.RunEvents = [];
BpodSystem.Data.RunSpeed = {};
BpodSystem.Data.CurrentTrialRunSpeed = zeros(10000,2);
BpodSystem.Data.CurrentTrialRunEvents = 0;
%% Initialize plots
BpodSystem.ProtocolFigures.SideOutcomePlotFig = figure('Position', [500 200 800 300],'name','Outcome plot','numbertitle','off', 'MenuBar', 'none', 'Resize', 'off');
BpodSystem.GUIHandles.SideOutcomePlot = axes('Position', [.075 .3 .89 .6]);
OneOutcomePlot(BpodSystem.GUIHandles.SideOutcomePlot,'init',TrialTypes);

% Set soft code handler to trigger sounds
BpodSystem.SoftCodeHandlerFunction = 'SoftCodeHandler_PlaySoundX';

T = timer('period',1,'executionmode','fixedrate','taskstoexecute',9999999);
T.TimerFcn = @(~,~)scriptMonitor();
T.Period = 1;
T.ExecutionMode = 'fixedRate';
start(T);
autoTimerCleanup = onCleanup(@()delete(timerfind));
%% Main trial loop
%Runs every second to monitor background events.

for currentTrial = 1:MaxTrials
    S = BpodParameterGUI('sync', S); % Sync parameters with BpodParameterGUI plugin
    sma = NewStateMatrix(); % Assemble state matrix
  %-----------------------------------------------------------------------------------  
        
        sma = SetGlobalTimer(sma,  'TimerID', 1, 'Duration',S.GUI.ImagingDuration, 'OnsetDelay', 0, 'Channel', 'BNC1');
        
        sma = AddState(sma, 'Name', 'TrialStart', ...%1
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'PreBaseline'},...
            'OutputActions', {'GlobalTimerTrig', 1});
        sma = AddState(sma, 'Name', 'PreBaseline', ...%2
            'Timer', S.GUI.BaselineTime,...
            'StateChangeConditions', {'Tup', 'LaserStimulation'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'LaserStimulation', ...%3 
            'Timer', S.GUI.LaserDuration,...
            'StateChangeConditions', {'Tup', 'PostBaseline'},...
            'OutputActions', {'BNC2',S.LaserStates});
        sma = AddState(sma, 'Name', 'PostBaseline', ...%4
            'Timer', S.GUI.BaselineTime,...
            'StateChangeConditions', {'Tup','ITI'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'ITI', ...%4
            'Timer', S.GUI.ITI,...
            'StateChangeConditions', {'Tup','exit'},...
            'OutputActions', {});
           %-----------------------------------------------------------------------------------  
    BpodSystem.ProtocolFigures.StartTime = 0;
    BpodSystem.Data.CurrentTrialRunSpeed = zeros(10000,2);
    BpodSystem.Data.CurrentTrialRunEvents = 0;
    BpodSystem.ProtocolFigures.CurrentTrialStartTime = datenum(datetime)*86400-BpodSystem.ProtocolFigures.StartTime;
    SendStateMatrix(sma);
    RawEvents = RunStateMatrix;
	BpodSystem.Data.LaserState(currentTrial) = S.LaserStates;
    if ~isempty(fieldnames(RawEvents)) % If trial data was returned
		%disp(RawEvents);
		RE = RawEvents.Events;
        %disp(RE);
		RES = RawEvents.States;
        %disp(RES);
		REST = RawEvents.StateTimestamps(2:end);
		%disp(REST);
		RET = RawEvents.EventTimestamps;
        %disp(RET);
		%Find if any timestamps contain a reward deliver (event 13) and
		%record that as the current trial reward delivery timestamp.
		RESR = (RES==13);
		REE = (RE==32);
% 		BpodSystem.Data.CurrentTrialLickTimes = round(nonzeros(REE.*RET)+BpodSystem.ProtocolFigures.CurrentTrialStartTime);
        %disp(BpodSystem.Data.CurrentTrialLickTimes)
        BpodSystem.Data = AddTrialEvents(BpodSystem.Data,RawEvents); % Computes trial events from raw data
        BpodSystem.Data.TrialSettings(currentTrial) = S; % Adds the settings used for the current trial to the Data struct (to be saved after the trial ends)
        BpodSystem.Data.TrialTypes(currentTrial) = TrialTypes(currentTrial); % Adds the trial type of the current trial to data
        BpodSystem.Data.ITI(currentTrial) = S.GUI.ITI;
        
        %Save the run timings and speeds
        BpodSystem.Data.RunEvents(currentTrial) = BpodSystem.Data.CurrentTrialRunEvents;
        BpodSystem.Data.RunSpeed{currentTrial} = BpodSystem.Data.CurrentTrialRunSpeed;
                
        UpdateOneOutcomePlot(TrialTypes, BpodSystem.Data);
        
        SaveBpodSessionData; % Saves the field BpodSystem.Data to the current data file
    end
    HandlePauseCondition; % Checks to see if the protocol is paused. If so, waits until user resumes.
    if BpodSystem.Status.BeingUsed == 0
        return
    end
end

function UpdateOneOutcomePlot(~, Data)
	         Outcomes = zeros(1,Data.nTrials);
	         for x = 1:Data.nTrials
		         Outcomes(x) = 1;
             end
             OneOutcomePlot(BpodSystem.GUIHandles.SideOutcomePlot,'update',Data.nTrials+1,TrialTypes,Outcomes);
end

end 
function scriptMonitor()
	%Runs every second to monitor background events. 
	global BpodSystem
    S = BpodSystem.ProtocolSettings;
	%Reads the data from the Arduino serial port. 
    BPAS = BpodSystem.PluginObjects.ardSerial;
	dataSize = get(BPAS,'BytesAvailable');
	if (dataSize>0)
		inASCII = fread(BPAS,dataSize);
		inData = transpose(char(inASCII));
		inData = strsplit(inData);
		inData = inData(1:end-1); % Remove the last newline from the array
		for k=1:length(inData) % Convert the array to run speed / run distance and write it to the run speed array. 
			BpodSystem.Data.CurrentTrialRunSpeed(k+BpodSystem.Data.CurrentTrialRunEvents,1:2) = str2double(strsplit(inData{k},'A'));
        end
        BpodSystem.Data.CurrentTrialRunEvents = BpodSystem.Data.CurrentTrialRunEvents + length(inData);
		assignin('base', 'inString', BpodSystem.Data.CurrentTrialRunEvents);
	end
% 	TotalRewardDisplay('update');
end
