%% Self stimulation task
% Xiong Xiao, 03/02/2017 @CSHL xiaoxiong2n@gmail.com
% Xian Zhang, modified on 09/27/2017 @CSHL gryllszhang@gmail.com
%
function SelfStimulation

% SETUP
% You will need:
% - A Bpod MouseBox (or equivalent) configured with 3 ports.
% > Connect the left port in the box to Bpod Port#5.
% > Connect the center port in the box to Bpod Port#x.
% > Connect the right port in the box to Bpod Port#6.
% > Make sure the liquid calibration tables for ports 1 and 3 have
%   calibration curves with several points surrounding 3ul.

global BpodSystem
global S
%% Define parameters
S = BpodSystem.ProtocolSettings; % Load settings chosen in launch manager into current workspace as a struct called S
if isempty(fieldnames(S))  % If settings file was an empty struct, populate struct with default settings    
    
    S.GUI.LaserSide = 1; % 1, left; 2, right
    S.GUI.LaserDuration = 1;
        
    S.GUI.TrainingLevel = 2; % case 1: Habituation; case 2: SelfStimulation (reward); case 3: SelfStimulation (aversive)
    S.GUIMeta.TrainingLevel.Style = 'popupmenu';
    S.GUIMeta.TrainingLevel.String = {'Habituation', 'SelfStimulation','Avoidance'};
    
    S.GUI.RewardAmount = 5;
    S.GUI.ResponseTime = 30;
    S.GUI.ITI = 0.1;

end

LeftValveState = 1; %1
RightValveState = 4; %4

% Initialize parameter GUI plugin
BpodParameterGUI('init', S);
TotalRewardDisplay('init');

%% Define trials
MaxTrials = 9999;
switch S.GUI.LaserSide
    case 1
        TrialTypes = ones(1,MaxTrials);
    case 2
        TrialTypes = 2*ones(1,MaxTrials);
end
BpodSystem.Data.TrialTypes = []; % The trial type of each trial completed will be added here.

%% Initialize plots
BpodSystem.ProtocolFigures.SideOutcomePlotFig = figure('Position', [200 200 1000 300],'name','Outcome plot','numbertitle','off', 'MenuBar', 'none', 'Resize', 'off');
BpodSystem.GUIHandles.SideOutcomePlot = axes('Position', [.075 .3 .89 .6]);
SideOutcomePlot(BpodSystem.GUIHandles.SideOutcomePlot,'init',2-TrialTypes);

%% Main trial loop
for currentTrial = 1:MaxTrials
    
    S = BpodParameterGUI('sync', S); % Sync parameters with BpodParameterGUI plugin
    % R = GetValveTimes(S.GUI.RewardAmount, [4 6]);
    R = [0.040,0.045];
    LeftValveTime = R(1); RightValveTime = R(2); % Update reward amounts
        
    switch TrialTypes(currentTrial) % Determine trial-specific state matrix fields
        case 1 % left port is rewarded
            ValveTime = LeftValveTime;
            ValveState = 1;
            resp_arg = 'Port1In';
        case 2 % right port is rewarded
            ValveTime = RightValveTime;
            ValveState = 4;
            resp_arg = 'Port3In';
    end
%--------------------------------------------------------------------------
    if S.GUI.TrainingLevel == 2
        if S.GUI.LaserSide==1
            laser_arg = {'Port1In','Laser','Port3In','ShamLaser','Tup','exit'};
        elseif S.GUI.LaserSide==2
            laser_arg = {'Port1In','ShamLaser','Port3In','Laser','Tup','exit'};
        end
    elseif S.GUI.TrainingLevel == 3
        if S.GUI.LaserSide==1
            laser_arg = {'Port1In','LaserValve','Port3In','ShamLaserValve','Tup','exit'};
            LaserValveState = 1;
            ShamLaserValveState = 4;
        elseif S.GUI.LaserSide==2
            laser_arg = {'Port1In','ShamLaserValve','Port3In','LaserValve','Tup','exit'};
            LaserValveState = 4;
            ShamLaserValveState = 1;
        end
    end
%------------------------------------------------------------------------
    sma = NewStateMatrix(); % Assemble state matrix
    switch S.GUI.TrainingLevel
        case 1 % Habituation
            sma = AddState(sma, 'Name', 'TrialStart', ...
                'Timer', 0.1,...
                'StateChangeConditions', {'Tup', 'WaitForResponse'},...
                'OutputActions', {});
            sma = AddState(sma, 'Name', 'WaitForResponse', ...
                'Timer', S.GUI.ResponseTime,...
                'StateChangeConditions', {'Port1In', 'LeftReward', 'Port3In', 'RightReward', 'Tup', 'ITI'},...
                'OutputActions', {});
            sma = AddState(sma, 'Name', 'LeftReward', ...
                'Timer', 0.05,...
                'StateChangeConditions', {'Tup', 'Drinking'},...
                'OutputActions', {'PWM1', 255,'ValveState', LeftValveState});
            sma = AddState(sma, 'Name', 'RightReward', ...
                'Timer', 0.05,...
                'StateChangeConditions', {'Tup', 'Drinking'},...
                'OutputActions', {'PWM3', 255,'ValveState', RightValveState});
            sma = AddState(sma, 'Name', 'Drinking', ...
                'Timer', 0,...
                'StateChangeConditions', {'Port1Out', 'ITI', 'Port3Out', 'ITI'},...
                'OutputActions', {});
            sma = AddState(sma, 'Name', 'ITI', ...
                'Timer', S.GUI.ITI,...
                'StateChangeConditions', {'Tup', 'exit'},...
                'OutputActions', {});
            
        case 2 % Self-Stimulation
                
            sma = AddState(sma, 'Name', 'TrialStart', ...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'WaitForResponse'},...
                'OutputActions', {});
            sma = AddState(sma, 'Name', 'WaitForResponse', ...
                'Timer', S.GUI.ResponseTime,...
                'StateChangeConditions', laser_arg,...
                'OutputActions', {});
            sma = AddState(sma, 'Name', 'Laser', ...
                'Timer', S.GUI.LaserDuration,...
                'StateChangeConditions', {'Tup', 'Drinking'},...
                'OutputActions', {'BNC2', 1});
            sma = AddState(sma, 'Name', 'ShamLaser', ...
                'Timer', S.GUI.LaserDuration,...
                'StateChangeConditions', {'Tup', 'Drinking'},...
                'OutputActions', {});
            sma = AddState(sma, 'Name', 'Drinking', ...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'exit'},...
                'OutputActions', {});  
            
            
        case 3 % avoidance
             sma = AddState(sma, 'Name', 'TrialStart', ...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'WaitForResponse'},...
                'OutputActions', {});
            sma = AddState(sma, 'Name', 'WaitForResponse', ...
                'Timer', S.GUI.ResponseTime,...
                'StateChangeConditions', laser_arg,...
                'OutputActions', {});
            sma = AddState(sma, 'Name', 'LaserValve', ...
                'Timer', 0.02,...
                'StateChangeConditions', {'Tup', 'Laser'},...
                'OutputActions', {'ValveState', LaserValveState});
            sma = AddState(sma, 'Name', 'ShamLaserValve', ...
                'Timer', 0.02,...
                'StateChangeConditions', {'Tup', 'ShamLaser'},...
                'OutputActions', {'ValveState', ShamLaserValveState});
            sma = AddState(sma, 'Name', 'Laser', ...
                'Timer', S.GUI.LaserDuration,...
                'StateChangeConditions', {'Tup', 'Drinking'},...
                'OutputActions', {'BNC2', 1});
            sma = AddState(sma, 'Name', 'ShamLaser', ...
                'Timer', S.GUI.LaserDuration,...
                'StateChangeConditions', {'Tup', 'Drinking'},...
                'OutputActions', {});
            sma = AddState(sma, 'Name', 'Drinking', ...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'exit'},...
                'OutputActions', {});     
            
    end
    
    SendStateMatrix(sma);
    RawEvents = RunStateMatrix;
    
    if ~isempty(fieldnames(RawEvents)) % If trial data was returned
        BpodSystem.Data = AddTrialEvents(BpodSystem.Data,RawEvents); % Computes trial events from raw data
        BpodSystem.Data.TrialSettings(currentTrial) = S; % Adds the settings used for the current trial to the Data struct (to be saved after the trial ends)
        BpodSystem.Data.TrialTypes(currentTrial) = TrialTypes(currentTrial); % Adds the trial type of the current trial to data
        
       if S.GUI.TrainingLevel == 1
               %Outcome
               if ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.Drinking(1))
                  BpodSystem.Data.Outcomes(currentTrial) = 1;
               else
                  BpodSystem.Data.Outcomes(currentTrial) = 0;
               end
            
       elseif S.GUI.TrainingLevel >= 2 
        
               if ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.Drinking(1))
                   BpodSystem.Data.Outcomes(currentTrial) = 1;
               elseif ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.Laser(1))
                   BpodSystem.Data.Outcomes(currentTrial) = 2;
               else
                   BpodSystem.Data.Outcomes(currentTrial) = 0;
               end
       end
       
              
      if S.GUI.TrainingLevel >= 2
       if ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.Laser(1))
           TrialTypes(currentTrial) = S.GUI.LaserSide;
       elseif ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.ShamLaser(1))
           TrialTypes(currentTrial) = 3-S.GUI.LaserSide;
       end
      end
           
        UpdateTotalRewardDisplay(S.GUI.RewardAmount, currentTrial);
        UpdateSideOutcomePlot(TrialTypes, BpodSystem.Data);
         
        SaveBpodSessionData; % Saves the field BpodSystem.Data to the current data file
    end
    HandlePauseCondition; % Checks to see if the protocol is paused. If so, waits until user resumes.
    if BpodSystem.Status.BeingUsed == 0
        return
    end
end

function UpdateSideOutcomePlot(TrialTypes, Data)
global BpodSystem
global S
Outcomes = zeros(1,Data.nTrials);
for x = 1:Data.nTrials   
  if S.GUI.TrainingLevel == 1
        if ~isnan(Data.RawEvents.Trial{x}.States.LeftReward(1))
            TrialTypes(x) = 1;
        elseif ~isnan(Data.RawEvents.Trial{x}.States.RightReward(1))
            TrialTypes(x) = 2;
        end    
        if ~isnan(Data.RawEvents.Trial{x}.States.Drinking(1))
            Outcomes(x) = 1;
        else
            Outcomes(x) = 0;
        end    
  elseif S.GUI.TrainingLevel >= 2  
  
    if ~isnan(Data.RawEvents.Trial{x}.States.Drinking(1))
        Outcomes(x) = 1;
    elseif ~isnan(Data.RawEvents.Trial{x}.States.Laser(1))
        Outcomes(x) = 2;
    else
        Outcomes(x) = 0;
    end
  end
end
SideOutcomePlot(BpodSystem.GUIHandles.SideOutcomePlot,'update',Data.nTrials+1,2-TrialTypes,Outcomes);

function UpdateTotalRewardDisplay(RewardAmount, currentTrial)
% If rewarded based on the state data, update the TotalRewardDisplay
global BpodSystem
if ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.Drinking(1))
    TotalRewardDisplay2('add', RewardAmount);
end

